import { parseJsonString } from "@egg/hooks"
import { z } from "zod"
import { zodToJsonSchema } from "zod-to-json-schema"

import { Context } from "../../types"
import llm from "../../utils/llm"
import { cleanLLMRes } from "../../utils/llm/base"

/**
 * 基础意图模式
 */
const BaseIntentSchema = z.object({
  intent: z.number().min(1).max(13),
})

/**
 * 简报模式
 */
const BriefingSchema = BaseIntentSchema.extend({
  intent: z.literal(5),
  link: z.string().url(),
  userDescription: z.string().min(1),
})

/**
 * 简报存储链接模式
 */
const BriefingLinkSchema = z.object({
  intent: z.literal(14),
  link: z.string().url(),
})

/**
 * 通用响应模式
 */
const CommonResponseSchema = BaseIntentSchema.extend({
  intent: z.literal(2),
  message: z.string().min(1),
})

/**
 * 联网检索模式
 */
const NetSearchSchema = BaseIntentSchema.extend({
  intent: z.literal(15),
  query: z.string().min(1),
})

/**
 * 意图模式
 */
const IntentSchema = z.union([
  BriefingSchema,
  CommonResponseSchema,
  BaseIntentSchema,
  BriefingLinkSchema,
  NetSearchSchema,
])

type Briefing = z.infer<typeof BriefingSchema>
type BriefingLink = z.infer<typeof BriefingLinkSchema>
type CommonResponse = z.infer<typeof CommonResponseSchema>
type NetSearch = z.infer<typeof NetSearchSchema>
export type Intent = z.infer<typeof IntentSchema>

/**
 * JSON模式
 */
const jsonSchema = zodToJsonSchema(IntentSchema)

/**
 * 代理函数
 * @param ctx - 上下文对象
 * @returns 返回意图对象
 */
const agent = async (ctx: Context): Promise<Intent> => {
  const {
    larkBody: { msgText },
    logger,
    requestId,
  } = ctx

  let attempts = 0
  while (attempts < 3) {
    const res = await llm.invoke(
      "intentRecognition",
      {
        userInput: msgText,
        time: new Date().toLocaleString("zh-CN", { timeZone: "Asia/Shanghai" }),
        jsonSchema,
      },
      requestId,
      0.5,
      true
    )

    const rawIntent = cleanLLMRes(res as string)
    const parsedIntent = parseJsonString(rawIntent, null)

    if (parsedIntent) {
      try {
        IntentSchema.parse(parsedIntent)
        logger.debug("Intent is valid: " + JSON.stringify(parsedIntent))
        return parsedIntent
      } catch (e: any) {
        logger.error("Invalid intent: " + String(e.errors))
      }
    } else {
      logger.error("Parsed intent is null")
    }

    attempts++
  }

  return {
    intent: 1,
  }
}

/**
 * 判断是否为简报意图
 * @param intent - 意图对象
 * @returns 如果是简报意图则返回true，否则返回false
 */
const isBriefing = (intent: Intent): intent is Briefing => {
  return intent.intent === 5
}

/**
 * 判断是否为简报链接意图
 * @param intent - 意图对象
 * @returns 如果是简报链接意图则返回true，否则返回false
 */
const isBriefingLink = (intent: Intent): intent is BriefingLink => {
  return intent.intent === 14
}

/**
 * 判断是否为通用响应意图
 * @param intent - 意图对象
 * @returns 如果是通用响应意图则返回true，否则返回false
 */
const isCommonResponse = (intent: Intent): intent is CommonResponse => {
  return intent.intent === 2
}

/**
 * 判断是否为联网检索意图
 * @param intent - 意图对象
 * @returns 如果是联网检索意图则返回true，否则返回false
 */
const isNetSearch = (intent: Intent): intent is NetSearch => {
  return intent.intent === 15
}

/**
 * 意图代理对象
 */
const intentAgent = {
  agent,
  isBriefing,
  isCommonResponse,
  isBriefingLink,
  isNetSearch,
}

export default intentAgent
